/* BBCIM: Acorn BBC computer diskimage manipulation
 *
 * archive_test.c
 * ENGLISH: reads file attributes from the inf/dsk files
 *
 * this file is public domain, by W.H.Scholten 2001
 */



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "archive_read_attributes.h"

int main(int argc, char **argv) {
	int  st;
	struct bbcfile file;
	struct bbcdiskimage disk;

	if (argc < 2) {
		fprintf(stderr, "use: test <filename>\n");
		exit(1);
	}

	st = bbcim_archive_read_file_attributes(argv[1], &file, 1);
	if (st < 0) {
		fprintf(stderr, "bbcim_archive_read_file_attributes returned error\n");
		bbcim_archive_free_file_attributes(&file);
	}
	else {
		int  i;
		char printable_name [ 16 ];
		int  printable_all = 1;
		strcpy(printable_name, file.name);
		for (i=0; i<strlen(file.name); i++) {
			if (printable_name[i] < 32 || printable_name[i] >= 127) {
				printable_name[i] = '?';
				printable_all = 0;
			}
		}

		printf("info gives: %s %s %6lX %6lX %s CRC=%4lX\n", printable_name, printable_all ? "" : " (contains unprintable characters) ", file.loadaddr, file.execaddr, ((file.attributes & BBCIM_ATTR__LOCKED) ? "Locked" : "Not locked"), file.crc);
		printf("file.type=%d\n", file.type);
		printf("file.attributes=%d\n", file.attributes);
		bbcim_archive_free_file_attributes(&file);
	}

	printf("\n");

	st = bbcim_archive_read_disk_attributes(argv[1], &disk, 1);
	if (st < 0) {
		fprintf(stderr, "bbcim_archive_read_disk_attributes returned error\n");
		//bbcim_archive_free_disk_attributes(&file);
	}
	else {
		printf(".dsk gives:\n"
			   " type=%d, sides=%d, tracks=%d, sec per track=%d, bps=%d\n", disk.type, disk.sides, disk.tracks, disk.sectors_per_track, disk.bytes_per_sector);
	}

	return 0;
}
