/* By W.H.Scholten 2001
 */

int decode_tape(char *tape_file, char *extract_directory, int tape_extract, int tape_catalogue) {
	char *tape_directory;
	FILE *fptape;
	FILE *fpblock;

	int  expect_new_file = True;
	unsigned char data[ 8192 ];
	int  next_id;
	int  nfiles = 0;

	int  block_counter = 0;

	char unix_filename[ 32 ];

	/* For NEXT <file> : */
	char prev_unix_filename [ 32 ];

	char filename[ 32 ];
	char printable_filename[ 32 ];


	unsigned int  load_addr;
	unsigned int  exec_addr;
	unsigned int  file_len;

	unsigned int  block_len;
	unsigned int  block_no;
	unsigned char block_flag;
	unsigned int  block_header_crc;
	unsigned int  block_data_crc;

	int  tape_block_no = 0;
	int  verbose = 0;

	prev_unix_filename [ 0 ] = 0;

	if (tape_extract) printf("Extracting files in %s:\n", tape_file);
	else if (tape_catalogue) printf("Showing files in %s:\n", tape_file);



	fptape = fopen(tape_file, "r");
	if (fptape == NULL) {
		fprintf(stderr, "Could not open tape file %s\n", tape_file);
		return -1;
	}

	tape_directory = strdup(tape_file);
	{
	char *slash = strrchr(tape_directory, '/');
	if (slash == NULL) strcpy(tape_directory, ".");
	else *slash = 0;
	}

	for (;;) {
		char tape_string[1024];
		int  ptr = ftell(fptape);
		char *st = fgets(tape_string, 1023, fptape);
		if (st) {
			DEB(printf("tape_string=%s\n", tape_string));
			if (tape_string[0]!='#') {
				fseek(fptape, ptr, SEEK_SET);
				break;
			}
		}
	}

	for(;;) {
		char block_filename[ 1024 ];
		//unsigned int  block_load_addr;
		//unsigned int  block_exec_addr;
		//unsigned int  block_flag;

		char string[1024];

		if (expect_new_file) {
			int  st;

			tape_block_no = 0;

			st = fscanf(fptape, "%s", string);
			DEB(printf("string = %s\n", string));
			if (st < 1) {
				DEB(printf("scanf st=%d\n", st));
				break; /* done or error */
			}
		} else tape_block_no++;

		sprintf(block_filename, "%s/%s_%02X", tape_directory, string, tape_block_no);
		DEB(printf("open file %s\n", block_filename));
		fpblock = fopen(block_filename, "r");
		if (fpblock == NULL) {
			fprintf(stderr, "unable to open %s\n", block_filename);
			expect_new_file = True;
			continue; /* next file */
		}
		fread(data, 1, 300, fpblock);


		/* Block id's left over from version based on 'uef_extract'  */
		next_id = 0x0100;

		switch(next_id) {
		case 0x0000:
			DEB(printf("origin block: %s\n", data));
			break;

		case 0x0100:
		{
			char *block_filename;
			unsigned char *d2;
			int  tmp_crc;

			DEB(printf("standard tape block\n"));

			block_filename = (char *) (data+1);
			if (expect_new_file) {
				nfiles++; /* Actually found a new file */
				file_len = 0;

				strcpy(filename, block_filename);

				/* Printable filename */
				strcpy(printable_filename, filename);
				/*file_allchars_printable =*/ bbcim_printable_bbcfilename(printable_filename);

				/* Local filename */
				strcpy(unix_filename, filename);
				bbcim_replace_bad_chars(unix_filename);
				bbcim_rename_sequential(unix_filename);

				DEB(printf("filename = %s\n", filename));

				if (tape_extract) {
					/* Add NEXT information for the previous file */
					if (strlen(prev_unix_filename) > 0) {
						FILE *fp;
						char inf_file[ 256 ];

						sprintf(inf_file, "%s/%s.inf", extract_directory, unix_filename);

						fp = fopen(inf_file, "a");
						if (fp) {
							fprintf(fp, " NEXT %s", unix_filename);
							fclose(fp);
						}
					}
				}
			}

			d2 = data + 1 + strlen(block_filename) + 1;
			load_addr = (*d2) | (*(d2+1) << 8) | (*(d2+2) << 16) | (*(d2+3) << 24);
			d2 += 4;
			exec_addr = (*d2) | (*(d2+1) << 8) | (*(d2+2) << 16) | (*(d2+3) << 24);
			d2 += 4;
			block_no = (*d2) | (*(d2+1) << 8);
			d2 += 2;
			block_len = (*d2) | (*(d2+1) << 8);
			d2 += 2;
			block_flag = (*d2);
			DEB(printf("block no=%d, block len=%d\n", block_no, block_len));
			d2 += 1+4; /* skip 4 spare bytes */

			/* 2 byte CRC (MSB): */
			block_header_crc = (*d2 << 8) | (*(d2+1));
			//DEB(printf("block header CRC=%4X\n", block_header_crc));
			d2 += 2;

			/* Check the header CRC */
			tmp_crc = bbcim_data_crc(data+1, strlen(filename)+1+19 -2);
			if (block_header_crc != tmp_crc) {
				fprintf(stderr, "%s  %02lX : header CRC check failed (calculated %4X, file %4X)\n", filename, block_counter, tmp_crc, block_header_crc);
			}
			else {
				DEB(printf("header CRC ok\n"));
				if (verbose) printf("%-10s  %02lX : header CRC ok: %4X\n", printable_filename, block_no, block_header_crc);
			}

			/* data: */
			if (tape_extract) {
				char bbc_file [256];
				FILE *fp;

				sprintf(bbc_file, "%s/%s", extract_directory, unix_filename);
				fp = fopen(bbc_file, "a");
				if (fp == NULL) {
					fprintf(stderr, "error appending file %s\n", bbc_file);
					exit(1);
				}

				fwrite(d2, 1, block_len, fp);
				fclose(fp);
			}
			d2 += block_len;

			file_len += block_len;

			/* 2 byte CRC (MSB), only if #bytes>0: */
			if (block_len > 0) {
				block_data_crc = (*d2 << 8) | (*(d2+1));
				//DEB(printf("block data CRC=%4X\n", block_data_crc));
				d2 += 2;

				/* Check the data CRC */
				tmp_crc = bbcim_data_crc(data+1+strlen(filename)+1+19 -2 +2, block_len);
				if (block_data_crc != tmp_crc) {
					fprintf(stderr, "%-10s  %02lX : data CRC check failed (calculated %4X, file %4X)\n", printable_filename, block_counter, tmp_crc, block_data_crc);
				}
				else {
					DEB(printf("data CRC ok\n"));
					if (verbose) printf("%-10s  %02lX : data CRC ok: %4X\n", printable_filename, block_no, block_data_crc);
				}
			}


			if (expect_new_file) {
				if (tape_extract) {
					/* Write .inf with load/exec from the first block, do not
					 * use those of e.g. the last block! */
					FILE *fp_info;
					char inf_file [256];

					sprintf(inf_file, "%s/%s.inf", extract_directory, unix_filename);

					fp_info = fopen(inf_file, "w");
					if (fp_info == NULL) {
						fprintf(stderr, "error opening .inf file\n");
						exit(1);
					}
					fprintf(fp_info, "TAPE %-10s %8X %8X CRC=%4X", filename, load_addr, exec_addr, bbcim_fcrc(unix_filename, 1));
					fclose(fp_info);
				}

				expect_new_file = False;
			}
			else {
				/* check for inconsistencies */
				if (strcmp(filename, block_filename)) {
					fprintf(stderr, "file? (%s -> %s)\n", printable_filename, block_filename);
				}
				/* Should also check change of load address/exec address */
			}

			if (block_flag & 0x80) {
				DEB(printf("done\n"));
				if (tape_catalogue) {
					printf("  %-10s %8X %8X %8X\n", printable_filename, load_addr, exec_addr, file_len);
				}
				expect_new_file = True;
				block_counter = 0;

				strcpy(prev_unix_filename, unix_filename);
			}
			else {
				block_counter++;
			}

			break;
			}

		case 0x0110:
		case 0x0111:
		case 0x0112:
			DEB(printf("tape data (hight tone/gap etc.)\n"));
			break;

		default:
			fprintf(stderr, "unhandled block %d\n", next_id);
			break;
		}
		DEB(printf("\n"));
	}
	free(tape_directory);

	if (nfiles == 0) printf("(No files found)\n");

	return 0;
}
