/* _bbcim_code.c
 *
 * By W.H.Scholten 1996-2001
 * This file is released to the public domain.
 */

/* Following code is from bbcim 0.99: */
char filename_chars_bad[]     = { '\\', '/', ':', '*', '?', '\"', '<', '>', '|', '.', '`', '\'' };
char filename_chars_replace[] = { '_' , '_', '_', '_', '_', '_' , '_', '_', '_', '_', '_', '_' };

/* Number of char's to be checked for replacement */
#define  no_filename_chars  sizeof(filename_chars_bad)

void bbcim_replace_bad_chars(char *unix_bbcfile_naam) {
	/* Name check: replace certain characters in filenames (most filesystems
	 * can't handle all characters from bbc filenames)
	 */
	int i, j;

	for (j=0; j<strlen(unix_bbcfile_naam); j++) {
		if (unix_bbcfile_naam[j] < 32 || unix_bbcfile_naam[j] > 126)
			unix_bbcfile_naam[j] = '_';
		else {
			for (i=0; i<no_filename_chars; i++) {
				if (unix_bbcfile_naam[j] == filename_chars_bad[i]) {
					unix_bbcfile_naam[j] = filename_chars_replace[i];
					break;
				}
			}
		}
	}
}


/* returns True if all chars are printable (>=32 and <=126), False if not;
 * Unprintable characters are replaced by '?' */
int bbcim_printable_bbcfilename(char *name) {
	int printable = true;
	int i;

	for (i=strlen(name)-1; i>=0; i--) {
		if (((unsigned char)name[i]) < 32 || ((unsigned char)name[i]) > 126) {
			name[i] = '?';
			printable = false;
		}
	}

	return printable;
}


long bbcim_fcrc(char *bestand, int verbose) {
	FILE *fp;
	unsigned long length;
	unsigned int crc;


	length = 0;
	crc = 0;

	fp = fopen(bestand, "rb");
	if (fp == NULL) return -1;
	while (1) {
		unsigned char filebyte;
		int  k;

		if (!fread(&filebyte, 1, 1, fp)) break;
		/* if (feof(fp)) break; */
		length++;
		crc ^= (filebyte << 8);
		for(k=0; k<8; k++) {
			if (crc & 32768) crc = (((crc ^ 0x0810) & 32767) << 1)+1;
			else crc <<= 1;
		}
	}
	fclose(fp);

	return crc;
}


long bbcim_data_crc(void *data, int length) {
	int  i;
	unsigned char *d = (unsigned char *) data;
	unsigned int crc;

	crc = 0;

	for (i=0; i<length; i++) {
		int  k;

		crc ^= ((*d++) << 8);
		for(k=0; k<8; k++) {
			if (crc & 32768) crc = (((crc ^ 0x0810) & 32767) << 1)+1;
			else crc <<= 1;
		}
	}

	return crc;
}


int bbcim_rename_sequential(char *name) {
	FILE *fp;
	static char new_name[ 100 ];
	int i = 0;

	strcpy(new_name, name);
	for(;;) {
		fp = fopen(new_name, "r");
		if (fp == NULL) break;
		fclose(fp);
		strcpy(new_name, name);
		sprintf(new_name+strlen(new_name), "%d", i);
		i++;
	}
	strcpy(name, new_name);

	return 1;
}
